
#include "enemy.h"

// Code redundancy is a Bad Thing.

CEnemy::CEnemy()
{
	Type = TypeList[ENEMY_GENERIC];
	Size = 4.5f;
	Rotation = CVector(0,randfloat()*360,0);
	Acceleration = CVector(0,0,0);
	Velocity = CVector(0,0,0);
	Location = CVector(0,0,0);
	HitPoints = 0;
	State = AI_IDLE_TRANS;

	Collided = false;
}

CEnemy::CEnemy(CVector Position, int HP)
{
	Type = TypeList[ENEMY_GENERIC];
	Size = 4.5f;
	Rotation = CVector(0,randfloat()*360,0);
	Acceleration = CVector(0,0,0);
	Velocity = CVector(0,0,0);
	SetPos(Position);
	HitPoints = HP;
	State = AI_IDLE_TRANS;

	Collided = false;
}

CEnemy::CEnemy(CVector Position, CVector Orientation, int HP)
{
	Type = TypeList[ENEMY_GENERIC];
	Size = 4.5f;
	Acceleration = Velocity = CVector(0,0,0);
	SetPos(Position);
	SetRot(Orientation);
	HitPoints = HP;
	State = AI_IDLE_TRANS;

	Collided = false;
}

CEnemy::~CEnemy()
{
}

void CEnemy::OnDraw(CCamera *camera)
{
	glTranslatef(Location.x, Location.y, Location.z);
	glRotatef(Rotation.y, 0,1,0);
	glColor3f(1,1,1);
	Rep->Draw();
}

void CEnemy::OnAnimate(float Time)
{
	Rep->SetAnimation(CurrentAnim, CurrentFrame, NextFrame, Percent);

	if(State != AI_DEAD)
	{
		Location += Velocity * Time;
		Location.y = Terrain->GetHeight(Location.x, Location.z) + 4.0f;
		
		Percent += Time* 3.0f;

		if(Percent >= 1.0f)
		{
			CurrentFrame++;
			if(CurrentFrame == CurrentAnim.EndFrame)
				CurrentFrame = CurrentAnim.StartFrame;
			NextFrame = CurrentFrame+1;
			if(NextFrame == CurrentAnim.EndFrame)
				NextFrame = CurrentAnim.StartFrame;
			Percent = 0.0f;
		}
	}
}

void CEnemy::OnAI()
{
	float FRand;

	PlayerLoc = CPlayer::Get()->GetLocation();
	PlayerVec = CVector(PlayerLoc.x - Location.x, 0.0f, PlayerLoc.z - Location.z);
	PlayerDist = PlayerVec.length();
	PlayerVec.norm(); 
	PlayerDir = -atan2f(PlayerVec.z, PlayerVec.x)*RADTODEG;

	switch(State)
	{
		case AI_IDLE_TRANS:
			SetAnimation(STAND);
			Velocity = CVector(0,0,0);
			State = AI_IDLE;
			break;

		case AI_IDLE:
			if(PlayerDist < 80.0f)
				State = AI_INTERESTED_TRANS;
			break;

		case AI_INTERESTED_TRANS:
			SetAnimation(RUN);
			State = AI_INTERESTED;
			break;

		case AI_INTERESTED:
			Rotation.y = PlayerDir;

			if(!Collided)
				Velocity = PlayerVec * 10.0f;
			else
				Velocity = CVector(0,0,0);

			if(PlayerDist > 90.0f)
				State = AI_IDLE_TRANS;
			break;

		/********** FAULTY ADD-IN SECTION: WATCH ***********/

		//Update:  Prelim tests of faulty segment are stable.

		case AI_DYING_TRANS:
			FRand = randfloat();
			SetAnimation((FRand < 0.3f) ? DEATH_ONE : 
			             (FRand < 0.7f) ? DEATH_TWO : DEATH_THREE);
			State = AI_DYING;		
			break;

		case AI_DYING:
			if(NextFrame == CurrentAnim.EndFrame-1) 
			{
				CurrentFrame = NextFrame = CurrentAnim.EndFrame-1;
				Percent = 100.0f;

				State = AI_DEAD;
			}
			break;

		case AI_DEAD:
			break;    

		/************** END FAULTY SECTION: *****************/

		default:
			break;
	};

	Collided = false;
}

void CEnemy::OnCollision(CObject *Collider)
{
	if(IsA(Collider->GetType(), TypeList[PROJECTILE_BULLET]))
	{
		Damage(30);
	}
	else if(IsA(Collider->GetType(), TypeList[ENTITY_PLAYER]))
	{
		CPlayer::Get()->Damage(50*CTimer::Get()->GetLastFrameTime());
		Velocity = CVector(0,0,0);
		Collided = true;
	}
	else if(IsA(Collider->GetType(), TypeList[ENEMY_GENERIC]))
	{
		//if anyone solves this, email me.


		/*if(Velocity.x > Velocity.z)
			Velocity = CVector(-Velocity.x, 0, Velocity.z);
		else
			Velocity = CVector(Velocity.x, 0, -Velocity.z);
		Collided = true;*/
	}


}

void CEnemy::SetAnimation(const Animation &NewAnim)
{
	CurrentAnim = NewAnim;
	CurrentFrame = NewAnim.StartFrame;
	NextFrame = NewAnim.StartFrame+1;
	Percent = 0.0f;
}

void CEnemy::Damage(int DamagePoints)
{
	if(HitPoints > 0)
	{
		HitPoints -= DamagePoints;
		if(HitPoints <= 0)
		{
			HitPoints = 0;
			State = AI_DYING_TRANS;
		}
	}
}

void CEnemy::SetPos(CVector NewPos)
{
	Location = NewPos;
}

void CEnemy::SetRot(CVector NewOri)
{
	Rotation = NewOri;
}

void CEnemy::SetVel(CVector NewVel)
{
	Velocity = NewVel;
}

void CEnemy::SetRep(CMD2Model *NewRep)
{
	Rep = NewRep;
}

void CEnemy::SetTerrain(CTerrain *NewTer)
{
	Terrain = NewTer;
}

