
#include "font.h"

CFont::CFont()
{
}

CFont::CFont(char *font_file, char *font_desc_file)
{
	Load(font_file, font_desc_file);
}

void CFont::Load(char *font_file, char *font_desc_file)
{
	TextureFile = font_file;
	DescFile = font_desc_file;

	Texture.Load(TextureFile);

	if(UtilityFileStream.is_open())
		UtilityFileStream.close();

	UtilityFileStream.open(font_desc_file, ios::in);

	UtilityFileStream >> CellPadding 
		              >> CellWidth >> CellHeight 
					  >> CharWidth >> CharHeight;
	UtilityFileStream.close();
}


CFont::~CFont()
{
}

void CFont::SetSize(unsigned int size)
{
	Size = size;
}

void CFont::SetColor(unsigned char red, unsigned char green, unsigned char blue, short alpha)
{
	Color[0] = red;
	Color[1] = green;
	Color[2] = blue;
	Alpha = unsigned char (alpha * 2.55f);
}

void CFont::DrawText(int x, int y, char *in_string, ...)
{
	char out_string[256];
	va_list params;
	va_start(params, in_string);
	vsprintf(out_string, in_string, params);
	va_end(params);

	Texture.Use();

	glPushAttrib(GL_COLOR_BUFFER_BIT);

	glLoadIdentity();

	glBlendFunc(GL_ONE,GL_ONE);
	glEnable(GL_BLEND);

	glColor4ub(Color[0], Color[1], Color[2], Alpha);

	glBegin(GL_QUADS);

	float raster_x = (float)x;
	float raster_y = (float)y;

	float RasterHeight = (float)Size;
	float RasterWidth = Size*(CharWidth/(float)CharHeight);

	int i = 0;

	while(out_string[i] != '\0')
	{		
		int CurrentChar = (int)out_string[i];

		float RasterTop    = raster_y+RasterHeight,
		      RasterBot    = raster_y,
        	  RasterLeft   = raster_x,
              RasterRight  = raster_x+RasterWidth,
		      TextureTop   = ((((CurrentChar/16)+1)*(CellHeight + CellPadding))/(float)-Texture.Height),
			  TextureBot   = TextureTop + (CharHeight/(float)Texture.Height),
			  TextureLeft  = (float(((CurrentChar%16)*(CellWidth + CellPadding))+CellPadding)/(float)Texture.Width),
			  TextureRight = TextureLeft + (CharWidth/(float)Texture.Width);

		glTexCoord2f(TextureRight, TextureTop);
		glVertex2f(RasterRight, RasterTop);

		glTexCoord2f(TextureRight, TextureBot);
		glVertex2f(RasterRight, RasterBot);

		glTexCoord2f(TextureLeft, TextureBot);
		glVertex2f(RasterLeft, RasterBot);

		glTexCoord2f(TextureLeft, TextureTop);
		glVertex2f(RasterLeft, RasterTop);

		raster_x += RasterWidth + 3;

		i++;
	}

	glEnd();

	glPopAttrib();
}
