#include "game.h"

CGame *CGame::Instance = 0;

GameState CGame::DisplayMenu()
{
	static CInputSystem *GameInput = CInputSystem::Get();

	static bool Load = true;

	if(Load)
	{
		Load = false;
		MenuMusic = CAudioSystem::Get()->LoadMP3("data/gui/menu.mp3");
		CAudioSystem::Get()->SetChannelVolume(MenuMusic, 60);
		CAudioSystem::Get()->PlayMP3(MenuMusic, false);

		MainMenu.Load("data/gui/main_menu.bmp");
		MenuCursor.Load("data/gui/cursor.bmp");
	}

	GameState ReturnVal = GAME_MENU_RUNNING;

	const int NumButtons = 4;
	RECT Button[NumButtons] = 
	//  {left,top,right,bottom}
	{
		{221,162,419,217}, 
		{196,245,434,291}, 
		{230,316,403,363}, 
		{526,399,571,426}
	};

	GameState ButtonState[NumButtons] = 
	{
		GAME_BRIEFING,
		GAME_LOADING,
		GAME_CREDITS,
		GAME_SHUTDOWN
	};

	static int MouseX = 0, MouseY = 0;
	int CurSize = 12;
	int MouseTempX, MouseTempY;
	GameInput->GetMouseMovement(MouseTempX, MouseTempY);
	MouseX -= MouseTempX; MouseY -= MouseTempY;
	if(MouseX > 640) MouseX = 640; if(MouseX < 0) MouseX = 0;
	if(MouseY > 480) MouseY = 480; if(MouseY < 0) MouseY = 0;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();

	GameEngine->SetRenderMode(CAFFEINE_ORTHO);

	glColor3f(1,1,1);

	MainMenu.Use();

	glBegin(GL_QUADS);
		glTexCoord2f(0,1);
		glVertex2i(0,0);
		glTexCoord2f(0,0);
		glVertex2i(0,480);
		glTexCoord2f(1,0);
		glVertex2f(640,480);
		glTexCoord2f(1,1);
		glVertex2i(640,0);
	glEnd();

	MenuCursor.Use();

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);

	glBegin(GL_QUADS);
		glTexCoord2f(0,1);
		glVertex2i(MouseX-CurSize,MouseY-CurSize);
		glTexCoord2f(0,0);
		glVertex2i(MouseX-CurSize,MouseY+CurSize);
		glTexCoord2f(1,0);
		glVertex2i(MouseX+CurSize,MouseY+CurSize);
		glTexCoord2f(1,1);
		glVertex2i(MouseX+CurSize,MouseY-CurSize);
	glEnd();


	glDisable(GL_BLEND);

	if(GameInput->MouseButtonDown(MOUSE_LEFT))
	{
		for(int i = 0; i < NumButtons; i++)
		{
			if(MouseX > Button[i].left && MouseX < Button[i].right &&
			   MouseY > Button[i].top && MouseY < Button[i].bottom)
			{
				ReturnVal =  ButtonState[i];
			}
		}
	}

	if(CInputSystem::Get()->KeyDown(VK_ESCAPE))
	{
		while(CInputSystem::Get()->KeyDown(VK_ESCAPE));
		return GAME_SHUTDOWN;
	}

	SwapBuffers(GameEngine->GetContext());

	if(ReturnVal != GAME_MENU_RUNNING)
		CAudioSystem::Get()->UnloadMP3(MenuMusic);

	return ReturnVal;
}

void CGame::LoadHUD()
{
	static bool Init = true;
	if(Init)
	{
		HUD.Load("data/gui/hud.bmp");
		HUDMask.Load("data/gui/hudmask.bmp");
		Crosshair.Load("data/gui/crosshair.bmp");
		CrosshairMask.Load("data/gui/crosshairmask.bmp");
		Init = false;
	}
}

void CGame::RenderHUD()
{
	GameFont->SetSize(23);

	glColor4ub(255,255,255,255);

	glEnable(GL_BLEND);

	glLoadIdentity();

	glBlendFunc(GL_ZERO, GL_ONE_MINUS_SRC_COLOR);

	HUDMask.Use();
	glBegin(GL_QUADS);
		glTexCoord2f(0.01f, 0.99f); glVertex2i(0, 416);
		glTexCoord2f(0.01f, 0.01f); glVertex2i(0, 480);
		glTexCoord2f(0.99f, 0.01f); glVertex2i(128, 480);
		glTexCoord2f(0.99f, 0.99f); glVertex2i(128, 416);
	glEnd();

	glBlendFunc(GL_ONE, GL_ONE);

	HUD.Use();
	glBegin(GL_QUADS);
		glTexCoord2f(0.01f, 0.99f); glVertex2i(0, 416);
		glTexCoord2f(0.01f, 0.01f); glVertex2i(0, 480);
		glTexCoord2f(0.99f, 0.01f); glVertex2i(128, 480);
		glTexCoord2f(0.99f, 0.99f); glVertex2i(128, 416);
	glEnd();

	int HealthCopy = CPlayer::Get()->GetHealth();

	if(HealthCopy < 30)
		glColor3f(1.0f, 0.0f, 0.0f);
	else if(HealthCopy < 60)
		glColor3f(1.0f, 1.0f, 0.0f);
	else 
		glColor3f(0.0f, 1.0f, 0.0f);
	glDisable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);
	glBegin(GL_TRIANGLE_FAN);
		glVertex2i(9, 439);
		glVertex2i(9, 466);
		glVertex2i(int(9+HealthCopy*0.45), 466);
		glVertex2i(int(9+HealthCopy*0.45), 439);
	glEnd();
	glEnable(GL_BLEND);
	glEnable(GL_TEXTURE_2D);

	int AmmoCopy = CPlayer::Get()->GetAmmo();
	//Display amount of ammo
	GameFont->SetColor(255,100,0,100);
	if(AmmoCopy < 10)
		GameFont->DrawText(80, 442, "0%i", AmmoCopy);
	else
		GameFont->DrawText(80, 442, "%i", AmmoCopy);


	glColor3f(1,1,1);

	int CrossSize = 12;

	glBlendFunc(GL_ZERO, GL_ONE_MINUS_SRC_COLOR);

	CrosshairMask.Use();
	glBegin(GL_QUADS);
		glTexCoord2f(0.01f, 0.99f); glVertex2i(320-CrossSize, 240-CrossSize);
		glTexCoord2f(0.01f, 0.01f); glVertex2i(320-CrossSize, 240+CrossSize);
		glTexCoord2f(0.99f, 0.01f); glVertex2i(320+CrossSize, 240+CrossSize);
		glTexCoord2f(0.99f, 0.99f); glVertex2i(320+CrossSize, 240-CrossSize);
	glEnd();

	glBlendFunc(GL_ONE, GL_ONE);

	Crosshair.Use();
	glBegin(GL_QUADS);
		glTexCoord2f(0.01f, 0.99f); glVertex2i(320-CrossSize, 240-CrossSize);
		glTexCoord2f(0.01f, 0.01f); glVertex2i(320-CrossSize, 240+CrossSize);
		glTexCoord2f(0.99f, 0.01f); glVertex2i(320+CrossSize, 240+CrossSize);
		glTexCoord2f(0.99f, 0.99f); glVertex2i(320+CrossSize, 240-CrossSize);
	glEnd();

	//DrawGun();

	glDisable(GL_BLEND);
}

/*void CGame::DrawGun()
{
	static CTexture Gun;
	static CTexture GunMask;
	static bool Init = true;
	if(Init)
	{
		Gun.Load("data/gui/gun2.bmp");
		GunMask.Load("data/gui/gun2mask.bmp");
		Init = false;
	}

	glColor4ub(255,255,255,255);

	glEnable(GL_BLEND);

	glLoadIdentity();

	glTranslatef(0,CPlayer::Get()->GetGunBob()*10.0f+10.0f, 0);

	glBlendFunc(GL_ZERO, GL_ONE_MINUS_SRC_COLOR);

	GunMask.Use();
	glBegin(GL_QUADS);
		glTexCoord2f(0.01f, 0.99f); glVertex2i(384, 300);
		glTexCoord2f(0.01f, 0.01f); glVertex2i(384, 480);
		glTexCoord2f(0.99f, 0.01f); glVertex2i(640, 480);
		glTexCoord2f(0.99f, 0.99f); glVertex2i(640, 300);
	glEnd();

	glBlendFunc(GL_ONE, GL_ONE);

	Gun.Use();
	glBegin(GL_QUADS);
		glTexCoord2f(0.01f, 0.99f); glVertex2i(384, 300);
		glTexCoord2f(0.01f, 0.01f); glVertex2i(384, 480);
		glTexCoord2f(0.99f, 0.01f); glVertex2i(640, 480);
		glTexCoord2f(0.99f, 0.99f); glVertex2i(640, 300);
	glEnd();

	glDisable(GL_BLEND);
}*/

bool CGame::HandleInput()
{
	static float mousesense = 0.2f;
	static CInputSystem *GameInput = CInputSystem::Get();
	static CPlayer *Player = CPlayer::Get();

	int MouseX, MouseY;

	GameInput->GetMouseMovement(MouseX, MouseY);

	Player->Rotate(MouseY*mousesense, MouseX*mousesense, 0.0f);

	if(GameInput->KeyDown(VK_W))
    	Player->Move(MOVE_FORWARD);

    if(GameInput->KeyDown(VK_A))
    	Player->Strafe(STRAFE_LEFT);

    if(GameInput->KeyDown(VK_S))
    	Player->Move(MOVE_BACKWARD);

    if(GameInput->KeyDown(VK_D))
    	Player->Strafe(STRAFE_RIGHT);

	if(GameInput->KeyDown(VK_SPACE))
		Player->Jump();

	if(GameInput->KeyDown(VK_LBUTTON))
		Player->Fire();

	if(GameInput->KeyDown(VK_ESCAPE))
		return false;

	if(GameInput->KeyDown(VK_ADD))
		mousesense += 0.1f;
	else if(GameInput->KeyDown(VK_SUBTRACT))
		if(mousesense > 0.1f)
			mousesense -= 0.1f;

	return true;
}

bool CGame::DisplayVictory(bool TheyWon)
{
	static float Dur = 0.0f;
	Dur += CTimer::Get()->GetLastFrameTime();
	if(Dur > 5.0f)
		return false;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	GameEngine->SetRenderMode(CAFFEINE_ORTHO);

	if(TheyWon)
	{
		GameFont->SetSize(40);
		GameFont->SetColor(0, 40, 90, 100);
		GameFont->DrawText(200, 240, "YOU WIN!");
		GameFont->SetColor(0, 0, 255, 100);
		GameFont->DrawText(202, 242, "YOU WIN!");
	}
	else
	{
		GameFont->SetSize(20);
		GameFont->SetColor(150, 0, 0, 100);
		GameFont->DrawText(250, 240, "YOU LOSE!");
	}

	SwapBuffers(GameEngine->GetContext());

	return true;
}


int CGame::RenderFrame()
{
	int ReturnVal = 0;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	GameEngine->SetRenderMode(CAFFEINE_PERSPECTIVE);

	CPlayer::Get()->RigCamera();

	ReturnVal = LevelOne.Draw(&GameCamera);

	CPlayer::Get()->Draw(&GameCamera);

	GameEngine->SetRenderMode(CAFFEINE_ORTHO);
	glPushMatrix();

	if(CPlayer::Get()->Underwater())
	{
		glLoadIdentity();
		glBlendFunc(GL_SRC_COLOR, GL_ONE);
		glEnable(GL_BLEND);
		glColor4ub(15,15,100,90);
		glDisable(GL_TEXTURE_2D);
		glBegin(GL_QUADS);
			glVertex2i(0,0);
			glVertex2i(0, 480); 
			glVertex2i(640, 480);
			glVertex2i(640, 0);
		glEnd();
		glDisable(GL_BLEND);
		glEnable(GL_TEXTURE_2D);
	}

	RenderHUD();
	//GameFont->DrawText(10, 35, "FPS: %f", CTimer::Get()->FPS());
	glPopMatrix();

	SwapBuffers(GameEngine->GetContext());

	return ReturnVal;
}

void CGame::Main()
{
	static GameState State = GAME_INIT;

	int Generic = 0;

	switch(State)
    {
        case GAME_INIT:
			randomize(CTimer::Get()->GetLastFrameTime()); // Bad Thing.
			LoadHUD();
            State = GAME_CUTSCENE_ONE_INIT;
            break;

		case GAME_CUTSCENE_ONE_INIT:
			CCutScene1::Get()->Load();
			State = GAME_CUTSCENE_ONE_PLAY;
			break;
		
		case GAME_CUTSCENE_ONE_PLAY:
			if(!CCutScene1::Get()->Draw())
				State = GAME_CUTSCENE_ONE_CLEANUP;
			break;

		case GAME_CUTSCENE_ONE_CLEANUP:
			CCutScene1::Get()->Unload();
			State = GAME_CUTSCENE_TWO_INIT;
			break;

		case GAME_CUTSCENE_TWO_INIT:
			CCutScene2::Get()->Load();
			State = GAME_CUTSCENE_TWO_PLAY;
			break;

		case GAME_CUTSCENE_TWO_PLAY:
			if(!CCutScene2::Get()->Draw())
				State = GAME_CUTSCENE_TWO_CLEANUP;
			break;

		case GAME_CUTSCENE_TWO_CLEANUP:
			CCutScene2::Get()->Unload();
			State = GAME_MENU_RUNNING;
			break;

		case GAME_MENU_RUNNING:
			State = DisplayMenu();
			break;

		case GAME_BRIEFING:
			Popup("    Thanks for playing!\n You can get your briefing\n in the file \"briefing.txt\"\n(I ran out of polish-time :)\n");
			State = GAME_MENU_RUNNING;
			break;

		case GAME_CREDITS:
			Popup("Credits can be found\nin \"credits.txt\"");
			State = GAME_MENU_RUNNING;
			break;

		case GAME_LOADING:
			GameFont = new CFont("data/fonts/trent.bmp", "data/fonts/trent.txt");
			CPlayer::Get()->SetCamera(&GameCamera);
			State = GAME_LEVEL_ONE_INIT;
			break;

		case GAME_LEVEL_ONE_INIT:
			LevelOne.Load("data/level1/level1.ini");
			CPlayer::Get()->SetTerrain(LevelOne.GetTerrain());
			//CurrentLevel = &LevelOne;
			State = GAME_RUNNING;
			break;

        case GAME_RUNNING:
			CTimer::Get()->Update();
			if(!HandleInput())
				State = GAME_SHUTDOWN;
			CPlayer::Get()->Animate(CTimer::Get()->GetLastFrameTime());
            
			Generic = RenderFrame();

			if(Generic == 0);
			else if(Generic == 1)
				State = GAME_DISPLAY_WINNER;
			else 
				State = GAME_DISPLAY_LOSER;
            break;

		case GAME_DISPLAY_WINNER:
			if(!DisplayVictory(true))
				State = GAME_SHUTDOWN;
			break;

		case GAME_DISPLAY_LOSER:
			if(!DisplayVictory(false))
				State = GAME_SHUTDOWN;
			break;

        case GAME_SHUTDOWN:
			LevelOne.Unload();
			PostQuitMessage(0);
            break;
     }
}

void CGame::Run()
{
	if(DialogBox(GetModuleHandle(0), MAKEINTRESOURCE(IDD_BOOTER),
		      0, (DLGPROC)BootupDlgProc) != IDOK)
	return;   // someone clicked the "EXIT" button!

	GameEngine = CEngine::Get(&BootData);

	ShowCursor(FALSE);

	for(ever)
	{
		if(GameEngine->Process() == CAFFEINE_EXIT)
			break;

		Main();

		CAudioSystem::Get()->Update();

		Sleep(1);    // necessary ...or evil? 
	}

	ShowCursor(TRUE);

	CAudioSystem::Get()->Kill();
}
 
int WINAPI WinMain(HINSTANCE, HINSTANCE, char*, int)
{
	CGame::Get()->Run();

	return 0;
}