
#ifndef CAFFEINE_OBJECT 
#define CAFFEINE_OBJECT 

#include "node.h"
#include "vector.h"
#include "camera.h"

#include "rti.h"  // implicitly included in all derived classes
                  // ...used for crude RTTI, but hey: it's fast :)

// THE METHODS of an OBJECT (AKA What I Learned From OpenGL Game Programming):
// *Draw
// *Animate
// *Perform AI
// *Collide

class CObject: public CNode
{
	protected:
		CVector Location;
		CVector Velocity;
		CVector Acceleration;

		CVector Rotation;

		float Size;

		RTT Type;  // This should be static.  Too bad that's impossible 
		           // ...because of inheritances

	public:
		virtual void OnDraw(CCamera *camera);
		virtual void OnAnimate(float Time);
		virtual void OnAI();
		virtual void OnCollision(CObject *);

		RTT GetType() { return Type; };

	public:
		CObject();
		~CObject();
 
		void Draw(CCamera *camera);
		void Animate(float Time);
		void AI();
		void Collide(CObject *);

		float GetSize() 
		{	return Size;	};

		CVector GetLocation()
		{	return Location;	};
};

#endif