#ifndef CAFFEINE_PARTICLE
#define CAFFEINE_PARTICLE

#include "default.h"
#include "constants.h"
#include "vector.h"
#include "texture.h"

typedef enum EmissionMode
{
	EMIT_EXPLOSION,
	EMIT_RESPAWN
};

struct PARTICLE_SYSTEM_DESC
{
	char Name[256];
	int MaxParticles;
	int EmissionRate;
	int Mode;
	CVector ColorStart;
	CVector ColorEnd;
	float AlphaStart, AlphaEnd;
	CVector Environment;
	float EmitterAngle;
	float SpeedBase, SpeedVary;
	float LifeBase, LifeVary;
	float SizeStart, SizeEnd;
	char TextureFileName[256];
	char MaskFileName[256];
	CTexture Texture;
	CTexture Mask;

};

bool LoadParticleSystem(char *File, PARTICLE_SYSTEM_DESC *psd);

class CParticle
{
	public:
		CVector Location;
		CVector Velocity;
		CVector Color;
		CVector ColorDelta;
		float Alpha;
		float Age;
		float Lifespan;
		float Size;
		bool Alive;

		CParticle() : Alive(false) {};
};

class CParticleSystem
{
	public:
		CParticleSystem() {};
		CParticleSystem(PARTICLE_SYSTEM_DESC *);
		~CParticleSystem();

		void Load(PARTICLE_SYSTEM_DESC *);
 
		void Update(float time);
		void Draw();
		void Position(CVector NewPos);

		void Start();
		void Stop();
		void Destroy();  // prematurly free all particles, or reinitialize

	private:
		CParticle *ParticleList;
		bool Emitting;
		int NumParticles;
		int MaxParticles;
		int EmissionRate;
		int Mode;
		CTexture *Texture;
		CTexture *Mask;
		CVector ColorStart;
		CVector ColorEnd;
		float AlphaStart;
		float AlphaEnd;
		float SizeStart;
		float SizeEnd;
		float LifeBase;
		float LifeVary;
		float SpeedBase;
		float SpeedVary;
		CVector Environment; // note: z not used
		CVector Emitter;
		float EmitterAngle;
};

#endif
