
#include "player.h"

/********** MISPLACED CBULLET DEFINITION **************/

void CBullet::Create(CVector Location, CVector Velocity, PARTICLE_SYSTEM_DESC &TrailDesc)
{
	Trail.Load(&TrailDesc);
	Trail.Position(CVector(0,0,0));
	this->Location = Location;
	this->Velocity = Velocity;

	LifeSpan = 3.0f;
	TimeAlive = 0.0f;
	IsAlive = true;
}

void CBullet::Kill()
{
	Trail.Destroy();	
	IsAlive = false;
}

void CBullet::OnDraw(CCamera *camera)
{
	glDisable(GL_TEXTURE_2D);
	DrawCube(Location);
	glEnable(GL_TEXTURE_2D);
	Trail.Draw();
}

void CBullet::OnAnimate(float Time)
{
	Trail.Position(Location);
	Trail.Update(Time);

	Location += Velocity*Time;
	TimeAlive += Time;
	if(TimeAlive > LifeSpan)
		IsAlive = false;
}


/*************** END CBULLET DEFINITION ****************/

void DrawCube(CVector Vc)
//I'm ashamed this wound up being the release DrawBullet() method...
{
	float Size = 0.2f;

	glColor3f(0.3f, 0.3f, 0.3f);

	glBegin(GL_QUADS);               
	//glColor3f(0.0f,1.0f,0.0f);
	glVertex3f(Vc.x+Size, Vc.y+Size, Vc.z-Size);    
	glVertex3f(Vc.x-Size, Vc.y+Size, Vc.z-Size);
	glVertex3f(Vc.x-Size, Vc.y+Size, Vc.z+Size);
	glVertex3f(Vc.x+Size, Vc.y+Size, Vc.z+Size);
 
	//glColor3f(1.0f,0.5f,0.0f);
	glVertex3f(Vc.x+Size, Vc.y-Size, Vc.z+Size);
	glVertex3f(Vc.x-Size, Vc.y-Size, Vc.z+Size);
	glVertex3f(Vc.x-Size, Vc.y-Size, Vc.z-Size);
	glVertex3f(Vc.x+Size, Vc.y-Size, Vc.z-Size);

	//glColor3f(1.0f,0.0f,0.0f);
	glVertex3f(Vc.x+Size, Vc.y+Size, Vc.z+Size);    
	glVertex3f(Vc.x-Size, Vc.y+Size, Vc.z+Size);    
	glVertex3f(Vc.x-Size, Vc.y-Size, Vc.z+Size);
	glVertex3f(Vc.x+Size, Vc.y-Size, Vc.z+Size);

	//glColor3f(1.0f,1.0f,0.0f);    
	glVertex3f(Vc.x+Size, Vc.y-Size, Vc.z-Size);
	glVertex3f(Vc.x-Size, Vc.y-Size, Vc.z-Size);
	glVertex3f(Vc.x-Size, Vc.y+Size, Vc.z-Size);
	glVertex3f(Vc.x+Size, Vc.y+Size, Vc.z-Size);

	//glColor3f(0.0f,0.0f,1.0f);    
	glVertex3f(Vc.x-Size, Vc.y+Size, Vc.z+Size);
	glVertex3f(Vc.x-Size, Vc.y+Size, Vc.z-Size);
	glVertex3f(Vc.x-Size, Vc.y-Size, Vc.z-Size);
	glVertex3f(Vc.x-Size, Vc.y-Size, Vc.z+Size);

	//glColor3f(1.0f,0.0f,1.0f);     
	glVertex3f(Vc.x+Size, Vc.y+Size, Vc.z-Size);
	glVertex3f(Vc.x+Size, Vc.y+Size, Vc.z+Size);
	glVertex3f(Vc.x+Size, Vc.y-Size, Vc.z+Size);
	glVertex3f(Vc.x+Size, Vc.y-Size, Vc.z-Size);
	glEnd();
}


CPlayer *CPlayer::Instance = 0;

void CPlayer::SetTerrain(CTerrain *NewTerrain)
{
	Terrain = NewTerrain;
}

void CPlayer::SetCamera(CCamera *NewCamera)
{
	Camera = NewCamera;
	Camera->Reset();
}

void CPlayer::Rotate(float Pitch, float Yaw, float Roll)
{
    Rotation.x -= Pitch;
	if(Rotation.x > 90.0f)
		Rotation.x = 90.0f;
	if(Rotation.x < -90.0f)
		Rotation.x = -90.0f;
    Rotation.y -= Yaw;
	if(Rotation.y > 180.0f)
		Rotation.y = -180.0f;
	if(Rotation.y < -180.0f)
		Rotation.y = 180.0f;
}

void CPlayer::Move(float Speed)
{
	if(Grounded  || Underwater())
	{
		Velocity.x += Speed*cosf(Rotation.y*DEGTORAD);
		Velocity.z += Speed*sinf(Rotation.y*DEGTORAD);
	}

	HeadBob = GetHeadBob();
}

void CPlayer::Strafe(float Speed)
{
	if(Grounded || Underwater())
	{
		Velocity.x += Speed*cosf((Rotation.y+90.0f)*DEGTORAD);
		Velocity.z += Speed*sinf((Rotation.y+90.0f)*DEGTORAD);
	}

	HeadBob = GetHeadBob();
}

void CPlayer::Position(CVector NewPosition)
{
	Location = NewPosition;
}

void CPlayer::Jump()
{
	if(Grounded)
	{
		Velocity.y += 16.0f;
		Grounded = false;
	}
	else if(Underwater() && Velocity.y < 22.0f)
	{
		Velocity.y += 8.0f;
		Grounded = false;
	}
}

int CPlayer::FindBulletSlot()
{
	for(int i = 0; i < BulletListSize; i++)
		if(!BulletList[i].Alive())
			return i;

	return -1;
}


void CPlayer::Fire()
{
	if(NumBullets <= 0 || RepTime > 0.0f)
		return;

	int FreeIndex = FindBulletSlot();
	
	if(FreeIndex == -1)
		return;

	CVector Pos;
	CVector Orient;
	CVector VelVec;

	Pos = CVector(Location.x, FinalYPos, Location.z);
	Orient = Rotation;

	float pitch = Orient.x;
	float yaw = Orient.y;
	float Vel = 600.0f;

	float fYSin = sinf(pitch*DEGTORAD);
	float fAbsYSin = sqrt(1-fYSin*fYSin);
	VelVec.x = cosf(yaw*DEGTORAD)*fAbsYSin * Vel;
	VelVec.y = -fYSin * Vel;
	VelVec.z = sinf(yaw*DEGTORAD)*fAbsYSin * Vel;
	float mag = sqrt(VelVec.x*VelVec.x + 
                     VelVec.y*VelVec.y + 
                     VelVec.z*VelVec.z);

	VelVec /= mag;
	VelVec *= Vel;

	BulletList[FreeIndex].Create(Pos, VelVec, FireDesc);

	RepTime = 0.5f;
	NumBullets--;
}

void CPlayer::OnAnimate(float Time)
{
	/*FireSystem.Position(BulletList[FreeIndex].Location);
	FireSystem.Update(Time);
	BulletList[FreeIndex].Location += BulletList[FreeIndex].Velocity*Time;*/

	for(int i = 0; i < BulletListSize; i++)
		if(BulletList[i].Alive())
			BulletList[i].Animate(Time);

	ApplyGravity(Time);
	
	Velocity+=Acceleration*Time;
	Location+=Velocity*Time;

	if(Location.z > 1200)
		Location.z = 1200;
	else if(Location.z < 80)
		Location.z = 80;

	if(Location.x > 1200)
		Location.x = 1200;
	else if(Location.x < 80)
		Location.x = 80;

	FinalYPos = Location.y+Height;	
	if(Grounded)
			FinalYPos += HeadBob;

	if(RepTime < Time)
		RepTime = 0.0f;
	else
		RepTime -= Time;
}

void CPlayer::OnDraw(CCamera *camera)
{
	for(int i = 0; i < BulletListSize; i++)
		if(BulletList[i].Alive())
			BulletList[i].Draw(camera);
	//FireSystem.Draw();
}

int CPlayer::GetHealth()
{
	return HitPoints;
}

int CPlayer::GetAmmo()
{
	return NumBullets;
}

void CPlayer::Damage(float DamagePoints)
{
	static float DamagePool = 0.0f;
	DamagePool += DamagePoints;

	if(HitPoints > 0 && DamagePool > 1)
	{
		HitPoints -= int(DamagePool);
		DamagePool = (float)fmod(DamagePool, (int)DamagePool);
		if(HitPoints <= 0)
			HitPoints = 0;
	}
}

void CPlayer::RigCamera()
{
	Camera->Position(Location.x, FinalYPos, Location.z);
	Camera->Orient(Rotation);
	Camera->Update();
}

bool CPlayer::Underwater()
{
	return FinalYPos < Terrain->GetWaterLine();
}

void CPlayer::ApplyGravity(float Delta)
{
	Acceleration.y = GRAVITY;

	if(!Grounded && Location.y > Terrain->GetHeight(Location.x, Location.z))
	{
		//Velocity -= Velocity*Delta*2.0f;
	}
	else
	{
		Location.y = Terrain->GetHeight(Location.x, Location.z);
		Velocity.y = 0.0f;
		Velocity -= Velocity*Delta*2.0f;  // freeze after 0.5 seconds
		Grounded = true;
	}
}

float CPlayer::GetHeadPos()
{
	return FinalYPos;
}

float CPlayer::GetHeadBob()
{
	HeadAngle += CTimer::Get()->GetLastFrameTime()*6.0f;
	if(HeadAngle >= PI)
		HeadAngle = -PI;
	return sinf(HeadAngle);
}

float CPlayer::GetGunBob()
{
	if(Grounded)
		return sinf(HeadAngle);
	else
		return 0.0f;
}
