
#include "terrain.h"

CTerrain::CTerrain(char *height_file, char *texture_file, char *detail_file)
{
	HeightFileName = height_file;
	Texture.Load(texture_file);
	DetailMap.Load(detail_file);

	/*IndexArray = new unsigned int[Width*Depth*6];
	TexCoordArray = new float[Width*Depth*2];

	for(int z = 0; z < Depth; z++)
		for(int x = 0; x < Width; x++)
		{
			CurVertex = z * Width + x;

			TexCoordArray[CurVertex*2] = float(x);
			TexCoordArray[CurVertex*2+1] = float(z);
		}

	for(z = 0; z < Depth; z++)
		for(int x = 0; x < Width; x++)
		{
			CurVertex = z * Width + x;
			IndexArray[index++] = CurVertex + Width;
			IndexArray[index++] = CurVertex;
		}

	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);

	glVertexPointer(3, GL_FLOAT, 0, TerrainData);
	glTexCoordPointer(2, GL_FLOAT, 0, TexCoordArray);*/

	unsigned char *imageData = LoadBitmap(HeightFileName, Width, Depth);

	Scale = 10;
	WaterHeight = 90;

	Terrain = new CVector[Width*Depth];

	for (int z = 0; z < Depth; z++)
	{
		for (int x = 0; x < Width; x++)
		{
			Terrain[x*Width+z].x = float(x)*Scale;				
			Terrain[x*Width+z].y = (float)imageData[(z*Depth+x)*3]*1.0f;
			Terrain[x*Width+z].z = float(z)*Scale;
		}
	}

	delete [] imageData;
}

CTerrain::CTerrain(char *height_file, char *texture_file, char* detail_file, 
			       float Scaling, float WaterTable)
{
	HeightFileName = height_file;
	Texture.Load(texture_file);
	DetailMap.Load(detail_file);

	unsigned char *imageData = LoadBitmap(HeightFileName, Width, Depth);

	Scale = Scaling;
	WaterHeight = WaterTable;

	Terrain = new CVector[Width*Depth];

	for (int z = 0; z < Depth; z++)
	{
		for (int x = 0; x < Width; x++)
		{
			Terrain[x*Width+z].x = float(x)*Scale;				
			Terrain[x*Width+z].y = (float)imageData[(z*Depth+x)*3]*1.0f;
			Terrain[x*Width+z].z = float(z)*Scale;
		}
	}

	delete [] imageData;
}

CTerrain::~CTerrain()
{
	if(Terrain)
		delete [] Terrain;

	/*if(TerrainData)
		delete [] TerrainData;
	TerrainData = 0;

	if(IndexArray)
		delete [] IndexArray;
	IndexArray = 0;

	if(TexCoordArray)
		delete [] TexCoordArray;
	TexCoordArray = 0;*/
}

void CTerrain::Draw(CCamera *camera)
{
	glColor3ub(255,255,255);

	static float MaxTexWidth = (float)Width;
	static float MaxTexHeight = (float)Depth;

	glActiveTexture(GL_TEXTURE0_ARB);
	glEnable(GL_TEXTURE_2D);
	Texture.Use();

	glActiveTexture(GL_TEXTURE1_ARB);
	glEnable(GL_TEXTURE_2D);
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_ARB);
	glTexEnvi(GL_TEXTURE_ENV, GL_RGB_SCALE_ARB, 2);
	DetailMap.Use();
	glMatrixMode(GL_TEXTURE);
		glLoadIdentity();
		glScalef(16.0f, 16.0f, 1);
	glMatrixMode(GL_MODELVIEW);

	int x;

	for (int z = 0; z < Depth-1; z++)
	{
		glBegin(GL_TRIANGLE_STRIP);

		x = 0;
		glTexCoord2f((x+1) / MaxTexWidth, (z+1) / MaxTexHeight);
		glVertex3f(Terrain[(x+1)*Width+(z+1)].x, 
			       Terrain[(x+1)*Width+(z+1)].y, 
				   Terrain[(x+1)*Width+(z+1)].z);

		for (x = 0; x < Width-1; x++)
		{
			float color = 0.7f;//Terrain[(x+1)*Width+z].y/255.0f;
			glColor3f(color, color, color);
			glMultiTexCoord2f(GL_TEXTURE0_ARB, x / MaxTexWidth, z / MaxTexHeight);
			glMultiTexCoord2f(GL_TEXTURE1_ARB, x / MaxTexWidth, z / MaxTexHeight);
			glVertex3f(Terrain[x*Width+z].x, 
				       Terrain[x*Width+z].y, 
					   Terrain[x*Width+z].z);

			glMultiTexCoord2f(GL_TEXTURE0_ARB, (x+1) / MaxTexWidth, z / MaxTexHeight);
			glMultiTexCoord2f(GL_TEXTURE1_ARB, (x+1) / MaxTexWidth, z / MaxTexHeight);
			glVertex3f(Terrain[(x+1)*Width+z].x, 
				       Terrain[(x+1)*Width+z].y, 
					   Terrain[(x+1)*Width+z].z);

			glMultiTexCoord2f(GL_TEXTURE0_ARB, x / MaxTexWidth, (z+1) / MaxTexHeight);
			glMultiTexCoord2f(GL_TEXTURE1_ARB, x / MaxTexWidth, (z+1) / MaxTexHeight);
			glVertex3f(Terrain[x*Width+(z+1)].x, 
				       Terrain[x*Width+(z+1)].y, 
					   Terrain[x*Width+(z+1)].z);

			glMultiTexCoord2f(GL_TEXTURE0_ARB, (x+1) / MaxTexWidth, (z+1) / MaxTexHeight);
			glMultiTexCoord2f(GL_TEXTURE1_ARB, (x+1) / MaxTexWidth, (z+1) / MaxTexHeight);
			glVertex3f(Terrain[(x+1)*Width+(z+1)].x, 
				       Terrain[(x+1)*Width+(z+1)].y, 
					   Terrain[(x+1)*Width+(z+1)].z);
		}

		glTexCoord2f(x / MaxTexWidth, z / MaxTexHeight);
		glVertex3f(Terrain[x*Width+z].x, 
			       Terrain[x*Width+z].y, 
				   Terrain[x*Width+z].z);
		glEnd();
	}

	glActiveTexture(GL_TEXTURE1_ARB);
	glDisable(GL_TEXTURE_2D);
	glActiveTexture(GL_TEXTURE0_ARB);
	glDisable(GL_TEXTURE_2D);

	glEnable(GL_TEXTURE_2D);

	static float Offset = 0.0f;
	static float Offset2 = 0.5f;

	Offset += 0.0015f;
	if(Offset > 1.0f)
		Offset = 0.0f;

	Offset2 += 0.0055f;
	if(Offset2 > 1.0f)
		Offset2 = 0.0f;

	static bool Load = true;
	if(Load)
	{
		Load = false;
		Water.Load("data/level1/water.bmp");
	}

	Water.Use();

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);

	glBegin(GL_QUADS);

	glColor4f(0.5f,0.5f, 1.0f, 1.0f);

		glTexCoord2f(20+Offset2,20); 
		glVertex3f(Width*Scale, WaterHeight-0.2f, Depth*Scale);

		glTexCoord2f(20+Offset2,0);  
		glVertex3f(Width*Scale, WaterHeight-0.2f, 0);

		glTexCoord2f(Offset2,0);     
		glVertex3f(0, WaterHeight-0.2f, 0);

		glTexCoord2f(Offset2,20);    
		glVertex3f(0, WaterHeight-0.2f, Depth*Scale);

	glColor4f(0.25f,0.25f, 0.5f, 1.0f);

		glTexCoord2f(20+Offset,20); 
		glVertex3f(Width*Scale, WaterHeight, Depth*Scale);

		glTexCoord2f(20+Offset,0);  
		glVertex3f(Width*Scale, WaterHeight, 0);

		glTexCoord2f(Offset,0);     
		glVertex3f(0, WaterHeight, 0);

		glTexCoord2f(Offset,20);    
		glVertex3f(0, WaterHeight, Depth*Scale);

	glEnd();

	glDisable(GL_BLEND);

	//for(int z = 0; z < Depth-1; z++)
	//	glDrawElements(GL_TRIANGLE_STRIP, Width*2, GL_UNSIGNED_INT, &IndexArray[z*Width*2]);
}

float CTerrain::GetWaterLine()
{
	return Scale * 9.0f;
}

// NOTE: CTerrain::GetHeight borrowed from SimpEngine, from OpenGL Game Programming
//       (http://glbook.gamedev.net); original code probably written by Kevin Hawkins,
//       Dave Astle, or Jason Shankel (in order of likelihood). Used by permission.

float CTerrain::GetHeight(float x, float z)
{	
	float tilex, tilez;

	tilex = (float)(x / Scale);
	tilez = (float)(z / Scale);

	int col0 = int(tilex);
	int row0 = int(tilez);
	int col1 = col0 + 1;
	int row1 = row0 + 1;

	if (col1 > Width*Scale)
		col1 = 0;
	if (row1 > Depth*Scale)
		row1 = 0;
                       
	float h00 = Terrain[col0*Width+row0].y;
	float h01 = Terrain[col1*Width+row0].y;
	float h11 = Terrain[col1*Width+row1].y;
	float h10 = Terrain[col0*Width+row1].y;

	float tx = tilex - float(col0);
	float ty = tilez - float(row0);
	float txty = tx * ty;

	float final_height = h00 * (1.0f - ty - tx + txty)
				       + h01 * (tx - txty)
					   + h11 * txty
                       + h10 * (ty - txty);

	return final_height;
}